<?php defined('SYSPATH') or die('No direct script access.');

/**
 * Class Model_Student
 *
 * @property-read $ID             int
 * @property-read $LastName       string
 * @property-read $FirstName      string
 * @property      $SecondName     string
 * @property-read $AccountID      int
 * @property-read $SemesterID     int
 * @property-read $GroupID        int
 * @property-read $GroupNum       int
 * @property      $GroupName      string
 * @property-read $GradeID        int
 * @property      $GradeNum       int
 * @property      $Degree         string
 * @property      $SpecID         int
 * @property      $SpecName       string
 * @property      $SpecAbbr       string
 * @property      $SpecCode       string
 * @property-read $FacultyID      int
 * @property      $FacultyName    string
 * @property      $FacultyAbbr    string
 * @property-read $ActivationCode string  #todo: add field
 */
class Model_Student extends Model_Container
{
    protected function getRawData($id) {
        $sql = 'CALL `Student_GetInfo`(:id)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $id)->execute();

        if ($info->count() == 0)
            throw new InvalidArgumentException('Student not found');
        return $info[0];
    }

    public static function make() {
        return new Model_Helper_StudentBuilder();
    }

    protected function create() {
        $sql = 'SELECT `CreateStudent`(LastName, FirstName, SecondName, GradeID, GroupNum, FacultyID, ActivationCode, SemesterID) AS `ID`';

        $this->data[self::$ID_FIELD] = DB::query(Database::SELECT, $sql)
            ->parameters($this->data)
            ->execute()->get('ID');

        if ($this->ID <= 0)
            throw new InvalidArgumentException(Error::INVALID_PARAMETERS);
    }

    /** @return Model_Discipline[] */
    public function getDisciplines($semesterID = null) {
        $semesterID = $semesterID ? $semesterID : User::instance()->SemesterID;
        $sql = 'CALL `GetDisciplinesForStudent`(:id, :semesterID)';
        $query = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semesterID', $semesterID)
            ->execute();

        $list = [];
        foreach ($query as $data)
            $list[] = new Model_Discipline($data, true);

        return $list;
    }

    public function getTeachers($loadAll = false, $semesterID = null) {
        // todo: don't load the full data at the Controller_Student_Index
        $semesterID = $semesterID ? $semesterID : User::instance()->SemesterID;

        $sql = 'CALL `GetTeachersListForStudent`(:id, :semester, :loadAll)';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semester', $semesterID)
            ->param(':loadAll', $loadAll)
            ->execute();

        $list = [];
        foreach ($result as $row) {
            $id = $row['DisciplineID'];
            $list[$id][] = $row;
        }
        return $list;
    }

    // todo implementation
    public function update() {
        throw new BadMethodCallException('Method is not implemented yet!');
    }

    /**
     * @see        students_groups.IsStudyLeave
     * @deprecated cause it's not implemented yet
     * @return boolean true, if the student is on leave
     */
    public function inAcademicLeave() {
        throw new BadMethodCallException('Method is not implemented yet!');
    }

    /**
     * Send the student to an academic leave.
     * @return $this;
     */
    public function toAcademicLeave($semesterID = null) {
        $semesterID = $semesterID ? $semesterID : User::instance()->SemesterID;
        $sql = 'CALL `ControlStudentGroup`(:id, -1, "leave", :semesterID)';


        DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semesterID', $semesterID)
            ->execute();

        return $this;
    }

    /**
     * Cancel the student's leave.
     * @param $group int  group id
     * @return $this;
     */
    public function returnFromAcademicLeave($group, $semesterID = null) {
        $semesterID = $semesterID ? $semesterID : User::instance()->SemesterID;;
        $sql = 'SELECT `ControlStudentGroup`(:id, :group, "common", :semesterID) AS Result';

        DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':group', (int) $group)
            ->param(':semesterID', $semesterID)
            ->execute();

        return $this;
    }

    public static function RecognizeDebtAndRepassing(Model_Discipline $discipline, $rates) {
        $res = [
            'debt'   => false,
            'repass' => false
        ];
        $disLimits = $discipline->getDisciplineRateInfo();
        $semester = $rates['Semester'] + $rates['PreviousExtra'];
        $exam     = max($rates['PrePreviousExam'], $rates['PreviousExam'], $rates['Exam']);

        $semesterRatingSetted = isset($rates['Semester']) || isset($rates['PreviousExtra']);
        $examRatingSetted = isset($rates['PrePreviousExam']) || isset($rates['PreviousExam']) ||  isset($rates['Exam']);

        if ($examRatingSetted || $semesterRatingSetted){
            $res['debt'] = ($semester + $exam) < 0.6 * $disLimits['Current'];
        }

        if ($discipline['Type'] == 'exam'){
            if ($examRatingSetted){
                $res['debt'] = $res['debt'] || $exam < 22;
                $res['repass'] = (int)($rates['PrePreviousExam']) < 22 && !$res['debt'];
            }
        }
        return $res;
    }
}
